require 'httparty'

module Feathr
  class Client
    include HTTParty

    attr_accessor :config

    def initialize(config: Feathr::Config.default)
      @config = config
    end

    def self.default
      @@default ||= Feathr::Client.new
    end

    def default_headers
      { 'Content-Type' => 'application/json' }
    end

    def auth_headers
      { Authorization: "Bearer #{ auth_token }" }
    end

    def auth_token
      @auth_token ||= if defined?(Rails)
        fetch_from_cache!
      else
        authenticate!
      end
    end

    def authenticate!
      @auth_token = api_token_auth.authenticate(
        email:    config.api_email,
        password: config.api_password
      )['token']
    end

    def refresh_token!
      @auth_token = api_token_refresh.refresh!(auth_token)['token']
    end

    def verify_token!
      api_token_verify.verify(auth_token)
    end

    def fetch_from_cache!
      Rails.cache.fetch('qwil_api_token', expires_in: 24.hours) { authenticate! }
    end

    def request(method: :get, path:, query: {}, headers: {})
      url = [config.base_url, path].join('/')



      unless path == Feathr::Api::ApiTokenAuth.new.api_path
        headers = default_headers.merge(auth_headers).merge(headers)
        query   = query.to_json
      end

      response = self.class.send(
        method,
        url,
        body:     query,
        headers:  headers
      )

      if response.success? && block_given?
        yield response.parsed_response
      elsif response.success?
        response.parsed_response
      else
        raise Feathr::Api::FeathrError.new(
          status: response.code,
          errors: response.parsed_response)
      end
    end

    def api_token_auth
      @api_token_auth ||= Feathr::Api::ApiTokenAuth.new(client: self)
    end

    def api_token_refresh
      @api_token_refresh ||= Feathr::Api::ApiTokenRefresh.new(client: self)
    end

    def api_token_verify
      @api_token_verify ||= Feathr::Api::ApiTokenVerify.new(client: self)
    end

    def reset_password
      @reset_password ||= Feathr::Api::ResetPassword.new(client: self)
    end

    def cashouts
      @cashouts ||= Feathr::Api::Cashouts.new(client: self)
    end

    def managers
      @managers ||= Feathr::Api::Managers.new(client: self)
    end

    def memberships
      @memberships ||= Feathr::Api::Memberships.new(client: self)
    end

    def platforms
      @platforms ||= Feathr::Api::Platforms.new(client: self)
    end

    def contractors
      @contractors ||= Feathr::Api::Contractors.new(client: self)
    end

    def incomes
      @incomes ||= Feathr::Api::Incomes.new(client: self)
    end

    def rebates
      @rebates ||= Feathr::Api::Rebates.new(client: self)
    end

    def receiveables
      @receiveables ||= Feathr::Api::Receiveables.new(client: self)
    end

    def featured_platforms
      @featured_platforms ||= Feathr::Api::FeaturedPlatforms.new(client: self)
    end

    def featured_platforms
      @public_platforms ||= Feathr::Api::PublicPlatforms.new(client: self)
    end

    def invoices
      @invoices ||= Feathr::Api::Invoices.new(client: self)
    end

    def memberships
      @memberships ||= Feathr::Api::Memberships.new(client: self)
    end

    def users
      @users ||= Feathr::Api::Users.new(client: self)
    end

  end
end
